@@ -22,19 +22,40 @@
 
 #include "config.h"
 
-#include "ostree.h"
-#include "otutil.h"
-
-#include <gio/gfiledescriptorbased.h>
-
 #include <sys/types.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <gio/gfiledescriptorbased.h>
 #include <attr/xattr.h>
+#include "ostree.h"
+#include "ostree-chain-input-stream.h"
+#include "otutil.h"
+#include "libgsystem.h"
 
 #define ALIGN_VALUE(this, boundary) \
   (( ((unsigned long)(this)) + (((unsigned long)(boundary)) -1)) & (~(((unsigned long)(boundary))-1)))
 
+/**
+ * SECTION:libostree-core
+ * @title: Core repository-independent functions
+ * @short_description: Create, validate, and convert core data types
+ *
+ * These functions implement repository-independent algorithms for
+ * operating on the core OSTree data formats, such as converting
+ * #GFileInfo into a #GVariant.
+ *
+ * There are 4 types of objects; file, dirmeta, tree, and commit.  The
+ * last 3 are metadata, and the file object is the only content object
+ * type.
+ *
+ * All metadata objects are stored as #GVariant (big endian).  The
+ * rationale for this is the same as that of the ext{2,3,4} family of
+ * filesystems; most developers will be using LE, and so it's better
+ * to continually test the BE->LE swap.
+ *
+ * The file object is a custom format in order to support streaming.
+ */
+
 const GVariantType *
 ostree_metadata_variant_type (OstreeObjectType objtype)
 {
@@ -51,6 +72,15 @@ ostree_metadata_variant_type (OstreeObjectType objtype)
     }
 }
 
+/**
+ * ostree_validate_checksum_string:
+ * @sha256: SHA256 hex string
+ * @error: Error
+ *
+ * Use this function to see if input strings are checksums.
+ *
+ * Returns: %TRUE if @sha256 is a valid checksum string, %FALSE otherwise
+ */
 gboolean
 ostree_validate_checksum_string (const char *sha256,
                                  GError    **error)
@@ -61,6 +91,19 @@ ostree_validate_checksum_string (const char *sha256,
 #define OSTREE_REF_FRAGMENT_REGEXP "[-._\\w\\d]+"
 #define OSTREE_REF_REGEXP "(?:" OSTREE_REF_FRAGMENT_REGEXP "/)*" OSTREE_REF_FRAGMENT_REGEXP
 
+/**
+ * ostree_parse_refspec:
+ * @refspec: A "refspec" string
+ * @out_remote: (out) (allow-none): The remote name, or %NULL if the refspec refs to a local ref
+ * @out_ref: (out) (allow-none): Name of ref
+ * @error: Error
+ *
+ * Split a refspec like "gnome-ostree:gnome-ostree/buildmaster" into
+ * two parts; @out_remote will be set to "gnome-ostree", and @out_ref
+ * will be "gnome-ostree/buildmaster".
+ *
+ * If @refspec refers to a local ref, @out_remote will be %NULL.
+ */
 gboolean
 ostree_parse_refspec (const char   *refspec,
                       char        **out_remote,
@@ -108,6 +151,13 @@ ostree_parse_refspec (const char   *refspec,
   return ret;
 }
 
+/**
+ * ostree_validate_rev:
+ * @rev: A revision string
+ * @error: Error
+ *
+ * Returns: %TRUE if @rev is a valid ref string
+ */
 gboolean
 ostree_validate_rev (const char *rev,
                      GError **error)
@@ -158,8 +208,10 @@ canonicalize_xattrs (char *xattr_string, size_t len)
     }
 
   xattrs = g_slist_sort (xattrs, (GCompareFunc) strcmp);
-  for (iter = xattrs; iter; iter = iter->next)
+  for (iter = xattrs; iter; iter = iter->next) {
     g_string_append (result, iter->data);
+    g_string_append_c (result, '\0');
+  }
 
   g_slist_free (xattrs);
   return g_string_free (result, FALSE);
@@ -180,28 +232,30 @@ read_xattr_name_array (const char *path,
     {
       ssize_t bytes_read;
       char *buf;
+      gs_unref_bytes GBytes *bytes = NULL;
 
       bytes_read = lgetxattr (path, p, NULL, 0);
       if (bytes_read < 0)
         {
           ot_util_set_error_from_errno (error, errno);
+          g_prefix_error (error, "lgetxattr (%s, %s) failed: ", path, p);
           goto out;
         }
       if (bytes_read == 0)
         continue;
 
       buf = g_malloc (bytes_read);
+      bytes = g_bytes_new_take (buf, bytes_read);
       if (lgetxattr (path, p, buf, bytes_read) < 0)
         {
           ot_util_set_error_from_errno (error, errno);
-          g_free (buf);
+          g_prefix_error (error, "lgetxattr (%s, %s) failed: ", path, p);
           goto out;
         }
       
       g_variant_builder_add (builder, "(@ay@ay)",
                              g_variant_new_bytestring (p),
-                             g_variant_new_from_data (G_VARIANT_TYPE ("ay"),
-                                                      buf, bytes_read, FALSE, g_free, buf));
+                             ot_gvariant_new_ay_bytes (bytes));
 
       p = p + strlen (p) + 1;
     }
@@ -211,6 +265,19 @@ read_xattr_name_array (const char *path,
   return ret;
 }
 
+/**
+ * ostree_get_xattrs_for_file:
+ * @f: a #GFile
+ * @out_xattrs: (out): A new #GVariant containing the extended attributes
+ * @cancellable: Cancellable
+ * @error: Error
+ *
+ * Read all extended attributes of @f in a canonical sorted order, and
+ * set @out_xattrs with the result.
+ *
+ * If the filesystem does not support extended attributes, @out_xattrs
+ * will have 0 elements, and this function will return successfully.
+ */
 gboolean
 ostree_get_xattrs_for_file (GFile         *f,
                             GVariant     **out_xattrs,
@@ -238,6 +305,7 @@ ostree_get_xattrs_for_file (GFile         *f,
       if (errno != ENOTSUP)
         {
           ot_util_set_error_from_errno (error, errno);
+          g_prefix_error (error, "llistxattr (%s) failed: ", path);
           goto out;
         }
     }
@@ -247,6 +315,7 @@ ostree_get_xattrs_for_file (GFile         *f,
       if (llistxattr (path, xattr_names, bytes_read) < 0)
         {
           ot_util_set_error_from_errno (error, errno);
+          g_prefix_error (error, "llistxattr (%s) failed: ", path);
           goto out;
         }
       xattr_names_canonical = canonicalize_xattrs (xattr_names, bytes_read);
@@ -298,6 +367,13 @@ ostree_file_header_new (GFileInfo         *file_info,
   return ret;
 }
 
+/**
+ * ostree_zlib_file_header_new:
+ * @file_info: a #GFileInfo
+ * @xattrs: (allow-none): Optional extended attribute array
+ *
+ * Returns: (transfer full): A new #GVariant containing file header for an archive-z2 repository
+ */
 GVariant *
 ostree_zlib_file_header_new (GFileInfo         *file_info,
                              GVariant          *xattrs)
@@ -366,6 +442,21 @@ write_padding (GOutputStream    *output,
   return ret;
 }
 
+/**
+ * ostree_write_variant_with_size:
+ * @output: Stream
+ * @variant: A variant
+ * @alignment_offset: Used to determine whether or not we should write padding bytes
+ * @out_bytes_written: (out): Number of bytes written
+ * @checksum: (allow-none): If provided, update with written data
+ * @cancellable: Cancellable
+ * @error: Error
+ *
+ * Use this function for serializing a chain of 1 or more variants
+ * into a stream; the @alignment_offset parameter is used to ensure
+ * that each variant begins on an 8-byte alignment so it can be safely
+ * accessed.
+ */
 gboolean
 ostree_write_variant_with_size (GOutputStream      *output,
                                 GVariant           *variant,
@@ -415,6 +506,17 @@ ostree_write_variant_with_size (GOutputStream      *output,
   return ret;
 }
 
+/**
+ * ostree_write_file_header_update_checksum:
+ * @out: Stream
+ * @variant: A variant, should be a file header
+ * @checksum: (allow-none): If provided, update with written data
+ * @cancellable: Cancellable
+ * @error: Error
+ *
+ * Write a file header variant to the provided @out stream, optionally
+ * updating @checksum.
+ */
 gboolean
 ostree_write_file_header_update_checksum (GOutputStream         *out,
                                           GVariant              *header,
@@ -434,6 +536,20 @@ ostree_write_file_header_update_checksum (GOutputStream         *out,
   return ret;
 }
 
+/**
+ * ostree_raw_file_to_content_stream:
+ * @input: File raw content stream
+ * @file_info: A file info
+ * @xattrs: (allow-none): Optional extended attributes
+ * @out_input: (out): Serialized object stream
+ * @out_length: (out): Length of stream
+ * @cancellable: Cancellable
+ * @error: Error
+ *
+ * Convert from a "bare" file representation into an
+ * OSTREE_OBJECT_TYPE_FILE stream.  This is a fundamental operation
+ * for writing data to an #OstreeRepo.
+ */
 gboolean
 ostree_raw_file_to_content_stream (GInputStream       *input,
                                    GFileInfo          *file_info,
@@ -483,6 +599,21 @@ ostree_raw_file_to_content_stream (GInputStream       *input,
   return ret;
 }
 
+/**
+ * ostree_content_stream_parse:
+ * @compressed: Whether or not the stream is zlib-compressed
+ * @input: Object content stream
+ * @input_length: Length of stream
+ * @trusted: If %TRUE, assume the content has been validated
+ * @out_input: (out): The raw file content stream
+ * @out_file_info: (out): Normal metadata 
+ * @out_xattrs: (out): Extended attributes
+ * @cancellable: Cancellable
+ * @error: Error
+ *
+ * The reverse of ostree_raw_file_to_content_stream(); this function
+ * converts an object content stream back into components.
+ */
 gboolean
 ostree_content_stream_parse (gboolean                compressed,
                              GInputStream           *input,
@@ -582,6 +713,20 @@ ostree_content_stream_parse (gboolean                compressed,
   return ret;
 }
 
+/**
+ * ostree_content_file_parse:
+ * @compressed: Whether or not the stream is zlib-compressed
+ * @content_path: Path to file containing content
+ * @trusted: If %TRUE, assume the content has been validated
+ * @out_input: (out): The raw file content stream
+ * @out_file_info: (out): Normal metadata 
+ * @out_xattrs: (out): Extended attributes
+ * @cancellable: Cancellable
+ * @error: Error
+ *
+ * A thin wrapper for ostree_content_stream_parse(); this function
+ * converts an object content stream back into components.
+ */
 gboolean
 ostree_content_file_parse (gboolean                compressed,
                            GFile                  *content_path,
@@ -606,11 +751,8 @@ ostree_content_file_parse (gboolean                compressed,
       if (!file_input)
         goto out;
       
-      if (fstat (g_file_descriptor_based_get_fd ((GFileDescriptorBased*)file_input), &stbuf) < 0)
-        {
-          ot_util_set_error_from_errno (error, errno);
-          goto out;
-        }
+      if (!gs_stream_fstat ((GFileDescriptorBased*)file_input, &stbuf, cancellable, error))
+        goto out;
 
       length = stbuf.st_size;
     }
@@ -644,6 +786,18 @@ ostree_content_file_parse (gboolean                compressed,
   return ret;
 }
 
+/**
+ * ostree_checksum_file_from_input:
+ * @file_info: File information
+ * @xattrs: (allow-none): Optional extended attributes
+ * @in: (allow-none): File content, should be %NULL for symbolic links
+ * @objtype: Object type
+ * @out_csum: (out) (array fixed-size=32): Return location for binary checksum
+ * @cancellable: Cancellable
+ * @error: Error
+ *
+ * Compute the OSTree checksum for a given input.
+ */
 gboolean
 ostree_checksum_file_from_input (GFileInfo        *file_info,
                                  GVariant         *xattrs,
@@ -697,6 +851,16 @@ ostree_checksum_file_from_input (GFileInfo        *file_info,
   return ret;
 }
 
+/**
+ * ostree_checksum_file:
+ * @f: File path
+ * @objtype: Object type
+ * @out_csum: (out) (array fixed-size=32): Return location for binary checksum
+ * @cancellable: Cancellable
+ * @error: Error
+ *
+ * Compute the OSTree checksum for a given file.
+ */
 gboolean
 ostree_checksum_file (GFile            *f,
                       OstreeObjectType  objtype,
@@ -774,6 +938,18 @@ checksum_file_async_data_free (gpointer datap)
   g_free (data);
 }
   
+/**
+ * ostree_checksum_file_async:
+ * @f: File path
+ * @objtype: Object type
+ * @io_priority: Priority for operation, see %G_IO_PRIORITY_DEFAULT
+ * @cancellable: Cancellable
+ * @callback: Invoked when operation is complete
+ * @user_data: Data for @callback
+ *
+ * Asynchronously compute the OSTree checksum for a given file;
+ * complete with ostree_checksum_file_async_finish().
+ */
 void
 ostree_checksum_file_async (GFile                 *f,
                             OstreeObjectType       objtype,
@@ -796,6 +972,16 @@ ostree_checksum_file_async (GFile                 *f,
   g_object_unref (res);
 }
 
+/**
+ * ostree_checksum_file_async_finish:
+ * @f: File path
+ * @result: Async result
+ * @out_csum: (out) (array fixed-size=32): Return location for binary checksum
+ * @error: Error
+ *
+ * Finish computing the OSTree checksum for a given file; see
+ * ostree_checksum_file_async().
+ */
 gboolean
 ostree_checksum_file_async_finish (GFile          *f,
                                    GAsyncResult   *result,
@@ -817,6 +1003,13 @@ ostree_checksum_file_async_finish (GFile          *f,
   return TRUE;
 }
 
+/**
+ * ostree_create_directory_metadata:
+ * @dir_info: a #GFileInfo containing directory information
+ * @xattrs: (allow-none): Optional extended attributes
+ *
+ * Returns: (transfer full): A new #GVariant containing %OSTREE_OBJECT_TYPE_DIR_META
+ */
 GVariant *
 ostree_create_directory_metadata (GFileInfo    *dir_info,
                                   GVariant     *xattrs)
@@ -833,6 +1026,17 @@ ostree_create_directory_metadata (GFileInfo    *dir_info,
   return ret_metadata;
 }
 
+/**
+ * ostree_set_xattrs:
+ * @f: a file
+ * @xattrs: Extended attribute list
+ * @cancellable: Cancellable
+ * @error: Error
+ *
+ * For each attribute in @xattrs, replace the value (if any) of @f for
+ * that attribute.  This function does not clear other existing
+ * attributes.
+ */
 gboolean
 ostree_set_xattrs (GFile  *f, 
                    GVariant *xattrs, 
@@ -858,11 +1062,12 @@ ostree_set_xattrs (GFile  *f,
                            &name, &value);
       value_data = g_variant_get_fixed_array (value, &value_len, 1);
       
-      loop_err = lsetxattr (path, (char*)name, (char*)value_data, value_len, XATTR_REPLACE) < 0;
+      loop_err = lsetxattr (path, (char*)name, (char*)value_data, value_len, 0) < 0;
       g_clear_pointer (&value, (GDestroyNotify) g_variant_unref);
       if (loop_err)
         {
           ot_util_set_error_from_errno (error, errno);
+          g_prefix_error (error, "lsetxattr (%s, %s) failed: ", path, name);
           goto out;
         }
     }
@@ -872,6 +1077,12 @@ ostree_set_xattrs (GFile  *f,
   return ret;
 }
 
+/**
+ * ostree_object_type_to_string:
+ * @objtype: an #OstreeObjectType
+ *
+ * Serialize @objtype to a string; this is used for file extensions.
+ */
 const char *
 ostree_object_type_to_string (OstreeObjectType objtype)
 {
@@ -891,6 +1102,12 @@ ostree_object_type_to_string (OstreeObjectType objtype)
     }
 }
 
+/**
+ * ostree_object_type_from_string:
+ * @str: A stringified version of #OstreeObjectType
+ *
+ * The reverse of ostree_object_type_to_string().
+ */
 OstreeObjectType
 ostree_object_type_from_string (const char *str)
 {
@@ -906,6 +1123,13 @@ ostree_object_type_from_string (const char *str)
   return 0;
 }
 
+/**
+ * ostree_object_to_string:
+ * @checksum: An ASCII checksum
+ * @objtype: Object type
+ *
+ * Returns: A string containing both @checksum and a stringifed version of @objtype
+ */
 char *
 ostree_object_to_string (const char *checksum,
                          OstreeObjectType objtype)
@@ -913,6 +1137,14 @@ ostree_object_to_string (const char *checksum,
   return g_strconcat (checksum, ".", ostree_object_type_to_string (objtype), NULL);
 }
 
+/**
+ * ostree_object_from_string:
+ * @str: An ASCII checksum
+ * @out_checksum: (out) (transfer full): Parsed checksum
+ * @out_objtype: (out): Parsed object type
+ *
+ * Reverse ostree_object_to_string().
+ */
 void
 ostree_object_from_string (const char *str,
                            gchar     **out_checksum,
@@ -926,6 +1158,12 @@ ostree_object_from_string (const char *str,
   *out_objtype = ostree_object_type_from_string (dot + 1);
 }
 
+/**
+ * ostree_hash_object_name:
+ * @a: A #GVariant containing a serialized object
+ *
+ * Use this function with #GHashTable and ostree_object_name_serialize().
+ */
 guint
 ostree_hash_object_name (gconstpointer a)
 {
@@ -939,6 +1177,13 @@ ostree_hash_object_name (gconstpointer a)
   return g_str_hash (checksum) + g_int_hash (&objtype_int);
 }
 
+/**
+ * ostree_cmp_checksum_bytes:
+ * @a: A binary checksum
+ * @b: A binary checksum
+ *
+ * Compare two binary checksums, using memcmp().
+ */
 int
 ostree_cmp_checksum_bytes (const guchar *a,
                            const guchar *b)
@@ -946,6 +1191,13 @@ ostree_cmp_checksum_bytes (const guchar *a,
   return memcmp (a, b, 32);
 }
 
+/**
+ * ostree_object_name_serialize:
+ * @checksum: An ASCII checksum
+ * @objtype: An object type
+ *
+ * Returns: (transfer floating): A new floating #GVariant containing checksum string and objtype
+ */
 GVariant *
 ostree_object_name_serialize (const char *checksum,
                               OstreeObjectType objtype)
@@ -955,6 +1207,15 @@ ostree_object_name_serialize (const char *checksum,
   return g_variant_new ("(su)", checksum, (guint32)objtype);
 }
 
+/**
+ * ostree_object_name_deserialize:
+ * @variant: A #GVariant of type (su)
+ * @out_checksum: (out) (transfer none): Pointer into string memory of @variant with checksum
+ * @out_objtype: (out): Return object type
+ *
+ * Reverse ostree_object_name_serialize().  Note that @out_checksum is
+ * only valid for the lifetime of @variant, and must not be freed.
+ */
 void
 ostree_object_name_deserialize (GVariant         *variant,
                                 const char      **out_checksum,
@@ -965,9 +1226,17 @@ ostree_object_name_deserialize (GVariant         *variant,
   *out_objtype = (OstreeObjectType)objtype_u32;
 }
 
-static void
-checksum_to_bytes (const char *checksum,
-                   guchar     *buf)
+/**
+ * ostree_checksum_inplace_to_bytes:
+ * @checksum: a SHA256 string
+ * @buf: Output buffer with at least 32 bytes of space
+ *
+ * Convert @checksum from a string to binary in-place, without
+ * allocating memory.  Use this function in hot code paths.
+ */
+void
+ostree_checksum_inplace_to_bytes (const char *checksum,
+                                  guchar     *buf)
 {
   guint i;
   guint j;
@@ -989,22 +1258,41 @@ checksum_to_bytes (const char *checksum,
     }
 }
 
+/**
+ * ostree_checksum_to_bytes:
+ * @checksum: An ASCII checksum
+ *
+ * Returns: (transfer full) (array fixed-size=32): Binary checksum from @checksum of length 32; free with g_free().
+ */
 guchar *
 ostree_checksum_to_bytes (const char *checksum)
 {
   guchar *ret = g_malloc (32);
-  checksum_to_bytes (checksum, ret);
+  ostree_checksum_inplace_to_bytes (checksum, ret);
   return ret;
 }
 
+/**
+ * ostree_checksum_to_bytes_v:
+ * @checksum: An ASCII checksum
+ *
+ * Returns: (transfer full): New #GVariant of type ay with length 32
+ */
 GVariant *
 ostree_checksum_to_bytes_v (const char *checksum)
 {
   guchar result[32];
-  checksum_to_bytes (checksum, result);
+  ostree_checksum_inplace_to_bytes (checksum, result);
   return ot_gvariant_new_bytearray ((guchar*)result, 32);
 }
 
+/**
+ * ostree_checksum_inplace_from_bytes: (skip)
+ * @csum: (array fixed-size=32): An binary checksum of length 32
+ * @buf: Output location, must be at least 65 bytes in length
+ *
+ * Overwrite the contents of @buf with stringified version of @csum.
+ */
 void
 ostree_checksum_inplace_from_bytes (const guchar *csum,
                                     char         *buf)
@@ -1021,6 +1309,12 @@ ostree_checksum_inplace_from_bytes (const guchar *csum,
   buf[j] = '\0';
 }
 
+/**
+ * ostree_checksum_from_bytes:
+ * @csum: (array fixed-size=32): An binary checksum of length 32
+ *
+ * Returns: (transfer full): String form of @csum
+ */
 char *
 ostree_checksum_from_bytes (const guchar *csum)
 {
@@ -1029,12 +1323,24 @@ ostree_checksum_from_bytes (const guchar *csum)
   return ret;
 }
 
+/**
+ * ostree_checksum_from_bytes_v:
+ * @csum_v: #GVariant of type ay
+ *
+ * Returns: (transfer full): String form of @csum_bytes
+ */
 char *
-ostree_checksum_from_bytes_v (GVariant *csum_bytes)
+ostree_checksum_from_bytes_v (GVariant *csum_v)
 {
-  return ostree_checksum_from_bytes (ostree_checksum_bytes_peek (csum_bytes));
+  return ostree_checksum_from_bytes (ostree_checksum_bytes_peek (csum_v));
 }
 
+/**
+ * ostree_checksum_bytes_peek:
+ * @bytes: #GVariant of type ay
+ *
+ * Returns: (transfer none): Binary checksum data in @bytes; do not free
+ */
 const guchar *
 ostree_checksum_bytes_peek (GVariant *bytes)
 {
@@ -1042,6 +1348,14 @@ ostree_checksum_bytes_peek (GVariant *bytes)
   return g_variant_get_fixed_array (bytes, &n_elts, 1);
 }
 
+/**
+ * ostree_get_relative_object_path:
+ * @checksum: ASCII checksum string
+ * @type: Object type
+ * @compressed: Whether or not the repository object is compressed
+ *
+ * Returns: (transfer full): Relative path for a loose object
+ */
 char *
 ostree_get_relative_object_path (const char         *checksum,
                                  OstreeObjectType    type,
@@ -1064,23 +1378,16 @@ ostree_get_relative_object_path (const char         *checksum,
   return g_string_free (path, FALSE);
 }
 
-char *
-ostree_get_relative_archive_content_path (const char        *checksum)
-{
-  GString *path;
-
-  g_assert (strlen (checksum) == 64);
-
-  path = g_string_new ("objects/");
-
-  g_string_append_len (path, checksum, 2);
-  g_string_append_c (path, '/');
-  g_string_append (path, checksum + 2);
-  g_string_append (path, ".filecontent");
-
-  return g_string_free (path, FALSE);
-}
-
+/**
+ * ostree_file_header_parse:
+ * @metadata: A metadata variant of type %OSTREE_FILE_HEADER_GVARIANT_FORMAT
+ * @out_file_info: (out): Parsed file information
+ * @out_xattrs: (out): Parsed extended attribute set
+ * @error: Error
+ *
+ * Load file header information into standard Gio #GFileInfo object,
+ * along with extended attributes tored in @out_xattrs.
+ */
 gboolean
 ostree_file_header_parse (GVariant         *metadata,
                           GFileInfo       **out_file_info,
@@ -1131,6 +1438,16 @@ ostree_file_header_parse (GVariant         *metadata,
   return ret;
 }
 
+/**
+ * ostree_zlib_file_header_parse:
+ * @metadata: A metadata variant of type %OSTREE_FILE_HEADER_GVARIANT_FORMAT
+ * @out_file_info: (out): Parsed file information
+ * @out_xattrs: (out): Parsed extended attribute set
+ * @error: Error
+ *
+ * Like ostree_file_header_parse(), but operates on zlib-compressed
+ * content.
+ */
 gboolean
 ostree_zlib_file_header_parse (GVariant         *metadata,
                                GFileInfo       **out_file_info,
@@ -1184,6 +1501,19 @@ ostree_zlib_file_header_parse (GVariant         *metadata,
   return ret;
 }
 
+/**
+ * ostree_create_file_from_input:
+ * @dest_file: Destination; must not exist
+ * @finfo: File information
+ * @xattrs: (allow-none): Optional extended attributes
+ * @input: (allow-none): Optional file content, must be %NULL for symbolic links
+ * @cancellable: Cancellable
+ * @error: Error
+ *
+ * Create a directory, regular file, or symbolic link, based on
+ * @finfo.  Append extended attributes from @xattrs if provided.  For
+ * %G_FILE_TYPE_REGULAR, set content based on @input.
+ */
 gboolean
 ostree_create_file_from_input (GFile            *dest_file,
                                GFileInfo        *finfo,
@@ -1220,9 +1550,21 @@ ostree_create_file_from_input (GFile            *dest_file,
     }
   else if (S_ISREG (mode))
     {
-      if (!gs_file_create (dest_file, mode, &out,
-                           cancellable, error))
-        goto out;
+      if (finfo != NULL)
+        {
+          uid = g_file_info_get_attribute_uint32 (finfo, "unix::uid");
+          gid = g_file_info_get_attribute_uint32 (finfo, "unix::gid");
+
+          if (!gs_file_create_with_uidgid (dest_file, mode, uid, gid, &out,
+                                           cancellable, error))
+            goto out;
+        }
+      else
+        {
+          if (!gs_file_create (dest_file, mode, &out,
+                               cancellable, error))
+            goto out;
+        }
 
       if (input)
         {
@@ -1233,6 +1575,16 @@ ostree_create_file_from_input (GFile            *dest_file,
 
       if (!g_output_stream_close ((GOutputStream*)out, NULL, error))
         goto out;
+
+      /* Work around libguestfs/FUSE bug */
+      if (mode & (S_ISUID|S_ISGID))
+        {
+          if (chmod (dest_path, mode) == -1)
+            {
+              ot_util_set_error_from_errno (error, errno);
+              goto out;
+            }
+        }
     }
   else if (S_ISLNK (mode))
     {
@@ -1250,7 +1602,10 @@ ostree_create_file_from_input (GFile            *dest_file,
       goto out;
     }
 
-  if (finfo != NULL)
+  /* We only need to chown for directories and symlinks; we already
+   * did a chown for files above via fchown().
+   */
+  if (finfo != NULL && !S_ISREG (mode))
     {
       uid = g_file_info_get_attribute_uint32 (finfo, "unix::uid");
       gid = g_file_info_get_attribute_uint32 (finfo, "unix::gid");
@@ -1278,6 +1633,22 @@ ostree_create_file_from_input (GFile            *dest_file,
   return ret;
 }
 
+/**
+ * ostree_create_temp_file_from_input:
+ * @dir: Target directory
+ * @prefix: Optional prefix
+ * @suffix: Optional suffix
+ * @finfo: File information
+ * @xattrs: (allow-none): Optional extended attributes
+ * @input: (allow-none): Optional file content, must be %NULL for symbolic links
+ * @out_file: (out): Path for newly created directory, file, or symbolic link
+ * @cancellable: Cancellable
+ * @error: Error
+ *
+ * Like ostree_create_file_from_input(), but securely allocates a
+ * randomly-named target in @dir.  This is a unified version of
+ * mkstemp()/mkdtemp() that also supports symbolic links.
+ */
 gboolean
 ostree_create_temp_file_from_input (GFile            *dir,
                                     const char       *prefix,
@@ -1339,37 +1710,17 @@ ostree_create_temp_file_from_input (GFile            *dir,
   return ret;
 }
 
-gboolean
-ostree_create_temp_regular_file (GFile            *dir,
-                                 const char       *prefix,
-                                 const char       *suffix,
-                                 GFile           **out_file,
-                                 GOutputStream   **out_stream,
-                                 GCancellable     *cancellable,
-                                 GError          **error)
-{
-  gboolean ret = FALSE;
-  gs_unref_object GFile *ret_file = NULL;
-  gs_unref_object GOutputStream *ret_stream = NULL;
-
-  if (!ostree_create_temp_file_from_input (dir, prefix, suffix, NULL, NULL, NULL,
-                                           &ret_file, cancellable, error))
-    goto out;
-  
-  if (out_stream)
-    {
-      ret_stream = (GOutputStream*)g_file_append_to (ret_file, 0, cancellable, error);
-      if (ret_stream == NULL)
-        goto out;
-    }
-  
-  ret = TRUE;
-  ot_transfer_out_value(out_file, &ret_file);
-  ot_transfer_out_value(out_stream, &ret_stream);
- out:
-  return ret;
-}
-
+/**
+ * ostree_create_temp_dir:
+ * @dir: Use this as temporary base
+ * @prefix: (allow-none): Optional prefix
+ * @suffix: (allow-none): Optional suffix
+ * @out_file: (out): Path for newly created directory, file, or symbolic link
+ * @cancellable: Cancellable
+ * @error: Error
+ *
+ * Securely create a randomly-named temporary subdirectory of @dir.
+ */
 gboolean
 ostree_create_temp_dir (GFile            *dir,
                         const char       *prefix,
@@ -1403,6 +1754,13 @@ ostree_create_temp_dir (GFile            *dir,
   return ret;
 }
 
+/**
+ * ostree_validate_structureof_objtype:
+ * @objtype:
+ * @error: Error
+ *
+ * Returns: %TRUE if @objtype represents a valid object type
+ */
 gboolean
 ostree_validate_structureof_objtype (guchar    objtype,
                                      GError   **error)
@@ -1418,6 +1776,13 @@ ostree_validate_structureof_objtype (guchar    objtype,
   return TRUE;
 }
 
+/**
+ * ostree_validate_structureof_csum_v:
+ * @checksum: a #GVariant of type "ay"
+ * @error: Error
+ *
+ * Returns: %TRUE if @checksum is a valid binary SHA256 checksum
+ */
 gboolean
 ostree_validate_structureof_csum_v (GVariant  *checksum,
                                     GError   **error)
@@ -1433,6 +1798,13 @@ ostree_validate_structureof_csum_v (GVariant  *checksum,
   return TRUE;
 }
 
+/**
+ * ostree_validate_structureof_checksum_string:
+ * @checksum: an ASCII string
+ * @error: Error
+ *
+ * Returns: %TRUE if @checksum is a valid ASCII SHA256 checksum
+ */
 gboolean
 ostree_validate_structureof_checksum_string (const char *checksum,
                                              GError   **error)
@@ -1484,6 +1856,16 @@ validate_variant (GVariant           *variant,
   return TRUE;
 }
 
+/**
+ * ostree_validate_structureof_commit:
+ * @commit: A commit object, %OSTREE_OBJECT_TYPE_COMMIT
+ * @error: Error
+ * 
+ * Use this to validate the basic structure of @commit, independent of
+ * any other objects it references.
+ *
+ * Returns: %TRUE if @commit is structurally valid
+ */
 gboolean
 ostree_validate_structureof_commit (GVariant      *commit,
                                     GError       **error)
@@ -1518,6 +1900,16 @@ ostree_validate_structureof_commit (GVariant      *commit,
   return ret;
 }
 
+/**
+ * ostree_validate_structureof_dirtree:
+ * @dirtree: A dirtree object, %OSTREE_OBJECT_TYPE_DIR_TREE
+ * @error: Error
+ * 
+ * Use this to validate the basic structure of @dirtree, independent of
+ * any other objects it references.
+ *
+ * Returns: %TRUE if @dirtree is structurally valid
+ */
 gboolean
 ostree_validate_structureof_dirtree (GVariant      *dirtree,
                                      GError       **error)
@@ -1586,6 +1978,13 @@ validate_stat_mode_perms (guint32        mode,
   return ret;
 }
 
+/**
+ * ostree_validate_structureof_file_mode:
+ * @mode: A Unix filesystem mode
+ * @error: Error
+ * 
+ * Returns: %TRUE if @mode represents a valid file type and permissions
+ */
 gboolean
 ostree_validate_structureof_file_mode (guint32            mode,
                                        GError           **error)
@@ -1607,6 +2006,15 @@ ostree_validate_structureof_file_mode (guint32            mode,
   return ret;
 }
 
+/**
+ * ostree_validate_structureof_dirmeta:
+ * @dirmeta: A dirmeta object, %OSTREE_OBJECT_TYPE_DIR_META
+ * @error: Error
+ * 
+ * Use this to validate the basic structure of @dirmeta.
+ *
+ * Returns: %TRUE if @dirmeta is structurally valid
+ */
 gboolean
 ostree_validate_structureof_dirmeta (GVariant      *dirmeta,
                                      GError       **error)
@@ -1634,3 +2042,19 @@ ostree_validate_structureof_dirmeta (GVariant      *dirmeta,
  out:
   return ret;
 }
+
+/**
+ * ostree_commit_get_parent:
+ * @commit_variant: Variant of type %OSTREE_OBJECT_TYPE_COMMIT
+ * 
+ * Returns: Binary checksum with parent of @commit_variant, or %NULL if none
+ */
+gchar *
+ostree_commit_get_parent (GVariant  *commit_variant)
+{
+  gs_unref_variant GVariant *bytes = NULL;
+  bytes = g_variant_get_child_value (commit_variant, 1);
+  if (g_variant_n_children (bytes) == 0)
+    return NULL;
+  return ostree_checksum_from_bytes_v (bytes);
+}
