@@ -22,16 +22,34 @@
 
 #pragma once
 
-#include <otutil.h>
+#include <gio/gio.h>
 
 G_BEGIN_DECLS
 
+/**
+ * OSTREE_MAX_METADATA_SIZE:
+ * 
+ * Maximum permitted size in bytes of metadata objects.
+ */
 #define OSTREE_MAX_METADATA_SIZE (1 << 26)
 
+/**
+ * OSTREE_MAX_RECURSION:
+ * 
+ * Maximum depth of metadata.
+ */
 #define OSTREE_MAX_RECURSION (256)
 
-#define OSTREE_EMPTY_STRING_SHA256 "e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855";
-
+/**
+ * OstreeObjectType:
+ * @OSTREE_OBJECT_TYPE_FILE: Content; regular file, symbolic link
+ * @OSTREE_OBJECT_TYPE_DIR_TREE: List of children (trees or files), and metadata
+ * @OSTREE_OBJECT_TYPE_DIR_META: Directory metadata
+ * @OSTREE_OBJECT_TYPE_COMMIT: Toplevel object, refers to tree and dirmeta for root
+ *
+ * Enumeration for core object types; %OSTREE_OBJECT_TYPE_FILE is for
+ * content, the other types are metadata.
+ */
 typedef enum {
   OSTREE_OBJECT_TYPE_FILE = 1,      /* .file */
   OSTREE_OBJECT_TYPE_DIR_TREE = 2,  /* .dirtree */
@@ -39,26 +57,44 @@ typedef enum {
   OSTREE_OBJECT_TYPE_COMMIT = 4     /* .commit */
 } OstreeObjectType;
 
+/**
+ * OSTREE_OBJECT_TYPE_IS_META:
+ * @t: An #OstreeObjectType
+ *
+ * Returns: %TRUE if object type is metadata
+ */
 #define OSTREE_OBJECT_TYPE_IS_META(t) (t >= 2 && t <= 4)
+
+/**
+ * OSTREE_OBJECT_TYPE_LAST:
+ *
+ * Last valid object type; use this to validate ranges.
+ */
 #define OSTREE_OBJECT_TYPE_LAST OSTREE_OBJECT_TYPE_COMMIT
 
-
-/*
- * file objects:
- * <BE guint32 containing variant length>
+/**
+ * OSTREE_FILE_HEADER_GVARIANT_FORMAT:
+ *
+ * File objects are stored as a stream, with one #GVariant header,
+ * followed by content.
+ * 
+ * The file header is of the following form:
+ *
+ * &lt;BE guint32 containing variant length&gt;
  * u - uid
  * u - gid
  * u - mode
  * u - rdev
  * s - symlink target 
  * a(ayay) - xattrs
- * ---
- * data
+ *
+ * Then the rest of the stream is data.
  */
 #define OSTREE_FILE_HEADER_GVARIANT_FORMAT G_VARIANT_TYPE ("(uuuusa(ayay))")
 
-/*
- * dirmeta objects:
+/**
+ * OSTREE_DIRMETA_GVARIANT_FORMAT:
+ *
  * u - uid
  * u - gid
  * u - mode
@@ -66,15 +102,17 @@ typedef enum {
  */
 #define OSTREE_DIRMETA_GVARIANT_FORMAT G_VARIANT_TYPE ("(uuua(ayay))")
 
-/*
- * Tree objects:
+/**
+ * OSTREE_TREE_GVARIANT_FORMAT:
+ *
  * a(say) - array of (filename, checksum) for files
  * a(sayay) - array of (dirname, tree_checksum, meta_checksum) for directories
  */
 #define OSTREE_TREE_GVARIANT_FORMAT G_VARIANT_TYPE ("(a(say)a(sayay))")
 
-/*
- * Commit objects:
+/**
+ * OSTREE_COMMIT_GVARIANT_FORMAT:
+ *
  * a{sv} - Metadata
  * ay - parent checksum (empty string for initial)
  * a(say) - Related objects
@@ -86,9 +124,13 @@ typedef enum {
  */
 #define OSTREE_COMMIT_GVARIANT_FORMAT G_VARIANT_TYPE ("(a{sv}aya(say)sstayay)")
 
-/*
- * filez objects:
- * <BE guint32 containing variant length>
+/**
+ * OSTREE_ZLIB_FILE_HEADER_GVARIANT_FORMAT:
+ *
+ * This is a variation on %OSTREE_FILE_HEADER_GVARIANT_FORMAT, used for
+ * storing zlib-compressed content objects.
+ *
+ * &lt;BE guint32 containing variant length&gt;
  * t - size
  * u - uid
  * u - gid
@@ -109,12 +151,15 @@ gboolean ostree_validate_checksum_string (const char *sha256,
 guchar *ostree_checksum_to_bytes (const char *checksum);
 GVariant *ostree_checksum_to_bytes_v (const char *checksum);
 
-char * ostree_checksum_from_bytes (const guchar *bytes);
-char * ostree_checksum_from_bytes_v (GVariant *bytes);
+char * ostree_checksum_from_bytes (const guchar *csum);
+char * ostree_checksum_from_bytes_v (GVariant *csum_v);
 
-void ostree_checksum_inplace_from_bytes (const guchar *bytes,
+void ostree_checksum_inplace_from_bytes (const guchar *csum,
                                          char         *buf);
 
+void ostree_checksum_inplace_to_bytes (const char *checksum,
+                                       guchar     *buf);
+
 const guchar *ostree_checksum_bytes_peek (GVariant *bytes);
 
 int ostree_cmp_checksum_bytes (const guchar *a, const guchar *b);
@@ -152,8 +197,6 @@ char *ostree_get_relative_object_path (const char        *checksum,
                                        OstreeObjectType   type,
                                        gboolean           compressed);
 
-char *ostree_get_relative_archive_content_path (const char        *checksum);
-
 gboolean ostree_get_xattrs_for_file (GFile         *f,
                                      GVariant     **out_xattrs,
                                      GCancellable  *cancellable,
@@ -180,11 +223,11 @@ GVariant *ostree_file_header_new (GFileInfo         *file_info,
 GVariant *ostree_zlib_file_header_new (GFileInfo         *file_info,
                                        GVariant          *xattrs);
 
-gboolean ostree_file_header_parse (GVariant         *data,
+gboolean ostree_file_header_parse (GVariant         *metadata,
                                    GFileInfo       **out_file_info,
                                    GVariant        **out_xattrs,
                                    GError          **error);
-gboolean ostree_zlib_file_header_parse (GVariant         *data,
+gboolean ostree_zlib_file_header_parse (GVariant         *metadata,
                                         GFileInfo       **out_file_info,
                                         GVariant        **out_xattrs,
                                         GError          **error);
@@ -210,7 +253,7 @@ gboolean ostree_content_file_parse (gboolean                compressed,
                                     GError                **error);
 
 gboolean ostree_write_file_header_update_checksum (GOutputStream         *out,
-                                                   GVariant              *header,
+                                                   GVariant              *variant,
                                                    GChecksum             *checksum,
                                                    GCancellable          *cancellable,
                                                    GError               **error);
@@ -232,7 +275,7 @@ gboolean ostree_checksum_file_from_input (GFileInfo        *file_info,
                                           GError          **error);
 
 gboolean ostree_checksum_file (GFile             *f,
-                               OstreeObjectType   type,
+                               OstreeObjectType   objtype,
                                guchar           **out_csum,
                                GCancellable      *cancellable,
                                GError           **error);
@@ -252,7 +295,7 @@ gboolean ostree_checksum_file_async_finish (GFile          *f,
 GVariant *ostree_create_directory_metadata (GFileInfo *dir_info,
                                             GVariant  *xattrs);
 
-gboolean ostree_create_file_from_input (GFile          *file,
+gboolean ostree_create_file_from_input (GFile          *dest_file,
                                         GFileInfo      *finfo,
                                         GVariant       *xattrs,
                                         GInputStream   *input,
@@ -269,14 +312,6 @@ gboolean ostree_create_temp_file_from_input (GFile            *dir,
                                              GCancellable     *cancellable,
                                              GError          **error);
 
-gboolean ostree_create_temp_regular_file (GFile            *dir,
-                                          const char       *prefix,
-                                          const char       *suffix,
-                                          GFile           **out_file,
-                                          GOutputStream   **out_stream,
-                                          GCancellable     *cancellable,
-                                          GError          **error);
-
 gboolean ostree_create_temp_dir (GFile            *dir,
                                  const char       *prefix,
                                  const char       *suffix,
@@ -298,14 +333,15 @@ gboolean ostree_validate_structureof_checksum_string (const char *checksum,
 gboolean ostree_validate_structureof_file_mode (guint32            mode,
                                                 GError           **error);
 
-gboolean ostree_validate_structureof_commit (GVariant      *index,
+gboolean ostree_validate_structureof_commit (GVariant      *commit,
                                              GError       **error);
 
-gboolean ostree_validate_structureof_dirtree (GVariant      *index,
+gboolean ostree_validate_structureof_dirtree (GVariant      *dirtree,
                                               GError       **error);
 
-gboolean ostree_validate_structureof_dirmeta (GVariant      *index,
+gboolean ostree_validate_structureof_dirmeta (GVariant      *dirmeta,
                                               GError       **error);
 
+gchar *  ostree_commit_get_parent            (GVariant  *commit_variant);
 
 G_END_DECLS
