@@ -22,6 +22,9 @@
 
 #include "config.h"
 
+#include <glib-unix.h>
+#include "otutil.h"
+
 #include "ostree-repo-file.h"
 #include "ostree-repo-private.h"
 
@@ -183,10 +186,7 @@ find_loose_for_checkout (OstreeRepo             *self,
       switch (self->mode)
         {
         case OSTREE_REPO_MODE_BARE:
-          path = ostree_repo_get_object_path (self, checksum, OSTREE_OBJECT_TYPE_FILE);
-          break;
-        case OSTREE_REPO_MODE_ARCHIVE:
-          path = ostree_repo_get_archive_content_path (self, checksum);
+          path = _ostree_repo_get_object_path (self, checksum, OSTREE_OBJECT_TYPE_FILE);
           break;
         case OSTREE_REPO_MODE_ARCHIVE_Z2:
           {
@@ -296,7 +296,6 @@ checkout_file_thread (GSimpleAsyncResult     *result,
   /* We can only do hardlinks in these scenarios */
   if (!is_symlink &&
       ((checkout_data->repo->mode == OSTREE_REPO_MODE_BARE && checkout_data->mode == OSTREE_REPO_CHECKOUT_MODE_NONE)
-       || (checkout_data->repo->mode == OSTREE_REPO_MODE_ARCHIVE && checkout_data->mode == OSTREE_REPO_CHECKOUT_MODE_USER)
        || (checkout_data->repo->mode == OSTREE_REPO_MODE_ARCHIVE_Z2 && checkout_data->mode == OSTREE_REPO_CHECKOUT_MODE_USER)))
     {
       if (!find_loose_for_checkout (checkout_data->repo, checksum, &loose_path,
@@ -759,6 +758,9 @@ ostree_repo_checkout_tree_finish (OstreeRepo               *self,
 
 /**
  * ostree_repo_checkout_gc:
+ * @self: Repo
+ * @cancellable: Cancellable
+ * @error: Error
  *
  * Call this after finishing a succession of checkout operations; it
  * will delete any currently-unused uncompressed objects from the
