@@ -20,7 +20,7 @@
  * Author: Colin Walters <walters@verbum.org>
  */
 
-/**
+/*
  * See:
  * https://mail.gnome.org/archives/ostree-list/2012-August/msg00021.html
  *
@@ -68,7 +68,9 @@
 #include "config.h"
 
 #include "ostree.h"
+#include "ostree-repo-private.h"
 #include "ostree-fetcher.h"
+#include "otutil.h"
 
 typedef struct {
   enum {
@@ -189,54 +191,34 @@ uri_fetch_update_status (gpointer user_data)
   OtPullData *pull_data = user_data;
   gs_free char *fetcher_status = NULL;
   GString *status;
-  guint64 current_bytes_transferred;
-  guint64 current_delta_bytes_transferred;
-  guint64 delta_bytes_transferred;
   guint outstanding_stages;
   guint outstanding_fetches;
  
   status = g_string_new ("");
 
-  if (!pull_data->metadata_scan_idle)
-    g_string_append_printf (status, "scan: %u metadata; ",
-                            g_atomic_int_get (&pull_data->n_scanned_metadata));
-
-  outstanding_stages = pull_data->n_outstanding_content_stage_requests + pull_data->n_outstanding_metadata_stage_requests;
-  if (outstanding_stages > 0)
-    g_string_append_printf (status, "writing: %u objects; ", outstanding_stages);
-
   outstanding_fetches = pull_data->n_outstanding_content_fetches + pull_data->n_outstanding_metadata_fetches;
+  outstanding_stages = pull_data->n_outstanding_content_stage_requests + pull_data->n_outstanding_metadata_stage_requests;
+
   if (outstanding_fetches)
     {
-      g_string_append_printf (status, "fetch: %u/%u metadata %u/%u content; ",
-                              pull_data->n_fetched_metadata,
-                              pull_data->n_requested_metadata,
-                              pull_data->n_fetched_content,
-                              pull_data->n_requested_content);
+      guint64 bytes_transferred = ostree_fetcher_bytes_transferred (pull_data->fetcher);
+      guint fetched = pull_data->n_fetched_metadata + pull_data->n_fetched_content;
+      guint requested = pull_data->n_requested_metadata + pull_data->n_requested_content;
+      gs_free char *formatted_bytes_transferred = NULL;
 
-      current_bytes_transferred = ostree_fetcher_bytes_transferred (pull_data->fetcher);
-      current_delta_bytes_transferred = current_bytes_transferred - pull_data->previous_total_downloaded;
+      formatted_bytes_transferred = g_format_size_full (bytes_transferred, 0);
 
-      if (pull_data->have_previous_bytes)
-        delta_bytes_transferred = (guint64)(0.5 * current_delta_bytes_transferred + 0.5 * pull_data->previous_bytes_sec);
-      else
-        {
-          pull_data->have_previous_bytes = TRUE;
-          delta_bytes_transferred = current_delta_bytes_transferred;
-        }
-      pull_data->previous_bytes_sec = delta_bytes_transferred;
-      pull_data->previous_total_downloaded = current_bytes_transferred;
-      
-      if (delta_bytes_transferred < 1024)
-        g_string_append_printf (status, "%u B/s; ", 
-                                (guint)delta_bytes_transferred);
-      else
-        g_string_append_printf (status, "%.1f KiB/s; ", 
-                                (double)delta_bytes_transferred / 1024);
-
-      fetcher_status = ostree_fetcher_query_state_text (pull_data->fetcher);
-      g_string_append (status, fetcher_status);
+      g_string_append_printf (status, "Receiving objects: %u%% (%u/%u) %s",
+                              (guint)((((double)fetched) / requested) * 100),
+                              fetched, requested, formatted_bytes_transferred);
     }
+  else if (outstanding_stages > 0)
+    g_string_append_printf (status, "Writing objects: %u", outstanding_stages);
+  else if (!pull_data->metadata_scan_idle)
+    g_string_append_printf (status, "Scanning metadata: %u",
+                            g_atomic_int_get (&pull_data->n_scanned_metadata));
+  else
+    g_string_append_printf (status, "Idle");
 
   gs_console_begin_status_line (gs_console_get (), status->str, NULL, NULL);
 
@@ -1179,7 +1161,7 @@ load_remote_repo_config (OtPullData    *pull_data,
 }
 
 gboolean
-ostree_repo_pull (OstreeRepo               *repo,
+ostree_repo_pull (OstreeRepo               *self,
                   const char               *remote_name,
                   char                    **refs_to_fetch,
                   OstreeRepoPullFlags       flags,
@@ -1219,7 +1201,7 @@ ostree_repo_pull (OstreeRepo               *repo,
   pull_data->loop = g_main_loop_new (pull_data->main_context, FALSE);
   pull_data->flags = flags;
 
-  pull_data->repo = repo;
+  pull_data->repo = self;
 
   pull_data->scanned_metadata = g_hash_table_new_full (ostree_hash_object_name, g_variant_equal,
                                                        (GDestroyNotify)g_variant_unref, NULL);
@@ -1231,10 +1213,10 @@ ostree_repo_pull (OstreeRepo               *repo,
   start_time = g_get_monotonic_time ();
 
   pull_data->remote_name = g_strdup (remote_name);
-  config = ostree_repo_get_config (repo);
+  config = ostree_repo_get_config (self);
 
   remote_key = g_strdup_printf ("remote \"%s\"", pull_data->remote_name);
-  if (!repo_get_string_key_inherit (repo, remote_key, "url", &baseurl, error))
+  if (!repo_get_string_key_inherit (self, remote_key, "url", &baseurl, error))
     goto out;
   pull_data->base_uri = soup_uri_new (baseurl);
 
@@ -1244,7 +1226,7 @@ ostree_repo_pull (OstreeRepo               *repo,
   if (tls_permissive)
     fetcher_flags |= OSTREE_FETCHER_FLAGS_TLS_PERMISSIVE;
 
-  pull_data->fetcher = ostree_fetcher_new (ostree_repo_get_tmpdir (pull_data->repo),
+  pull_data->fetcher = ostree_fetcher_new (pull_data->repo->tmp_dir,
                                            fetcher_flags);
 
   if (!pull_data->base_uri)
@@ -1379,24 +1361,11 @@ ostree_repo_pull (OstreeRepo               *repo,
       gs_free char *key = NULL;
       gs_free char *remote_ref = NULL;
       gs_free char *baseurl = NULL;
-      gs_free char *original_rev = NULL;
 
-      remote_ref = g_strdup_printf ("%s/%s", pull_data->remote_name, ref);
-
-      if (!ostree_repo_resolve_rev (pull_data->repo, remote_ref, TRUE, &original_rev, error))
-        goto out;
-
-      if (original_rev && strcmp (sha256, original_rev) == 0)
-        {
-          g_print ("No changes in %s\n", remote_ref);
-        }
-      else
-        {
-          ot_waitable_queue_push (pull_data->metadata_objects_to_scan,
-                                  pull_worker_message_new (PULL_MSG_SCAN,
-                                                           ostree_object_name_serialize (sha256, OSTREE_OBJECT_TYPE_COMMIT)));
-          g_hash_table_insert (updated_refs, g_strdup (ref), g_strdup (sha256));
-        }
+      ot_waitable_queue_push (pull_data->metadata_objects_to_scan,
+                              pull_worker_message_new (PULL_MSG_SCAN,
+                                                       ostree_object_name_serialize (sha256, OSTREE_OBJECT_TYPE_COMMIT)));
+      g_hash_table_insert (updated_refs, g_strdup (ref), g_strdup (sha256));
     }
   
   {
@@ -1424,13 +1393,24 @@ ostree_repo_pull (OstreeRepo               *repo,
       const char *ref = key;
       const char *checksum = value;
       gs_free char *remote_ref = NULL;
+      gs_free char *original_rev = NULL;
           
       remote_ref = g_strdup_printf ("%s/%s", pull_data->remote_name, ref);
-          
-      if (!ostree_repo_write_ref (pull_data->repo, pull_data->remote_name, ref, checksum, error))
+
+      if (!ostree_repo_resolve_rev (pull_data->repo, remote_ref, TRUE, &original_rev, error))
         goto out;
           
-      g_print ("remote %s is now %s\n", remote_ref, checksum);
+      if (original_rev && strcmp (checksum, original_rev) == 0)
+        {
+          g_print ("remote %s is unchanged from %s\n", remote_ref, original_rev);
+        }
+      else
+        {
+          if (!ostree_repo_write_ref (pull_data->repo, pull_data->remote_name, ref, checksum, error))
+            goto out;
+          
+          g_print ("remote %s is now %s\n", remote_ref, checksum);
+        }
     }
       
   end_time = g_get_monotonic_time ();
