@@ -41,9 +41,16 @@ gboolean      ostree_repo_check (OstreeRepo  *self, GError **error);
 
 GFile *       ostree_repo_get_path (OstreeRepo  *self);
 
+/**
+ * OstreeRepoMode:
+ * @OSTREE_REPO_MODE_BARE: Files are stored as themselves; can only be written as root
+ * @OSTREE_REPO_MODE_ARCHIVE_Z2: Files are compressed, should be owned by non-root.  Can be served via HTTP
+ *
+ * See the documentation of #OstreeRepo for more information about the
+ * possible modes.
+ */
 typedef enum {
   OSTREE_REPO_MODE_BARE,
-  OSTREE_REPO_MODE_ARCHIVE,
   OSTREE_REPO_MODE_ARCHIVE_Z2
 } OstreeRepoMode;
 
@@ -53,8 +60,6 @@ gboolean       ostree_repo_mode_from_string (const char      *mode,
 
 OstreeRepoMode ostree_repo_get_mode (OstreeRepo  *self);
 
-GFile *       ostree_repo_get_tmpdir (OstreeRepo  *self);
-
 GKeyFile *    ostree_repo_get_config (OstreeRepo *self);
 
 GKeyFile *    ostree_repo_copy_config (OstreeRepo *self);
@@ -65,16 +70,6 @@ gboolean      ostree_repo_write_config (OstreeRepo *self,
                                         GKeyFile   *new_config,
                                         GError    **error);
 
-GFile *       ostree_repo_get_object_path (OstreeRepo   *self,
-                                           const char   *object,
-                                           OstreeObjectType type);
-
-GFile *       ostree_repo_get_archive_content_path (OstreeRepo    *self,
-                                                    const char    *checksum);
-
-GFile *       ostree_repo_get_file_object_path (OstreeRepo   *self,
-                                                const char   *object);
-
 gboolean      ostree_repo_prepare_transaction (OstreeRepo     *self,
                                                gboolean        enable_commit_hardlink_scan,
                                                gboolean       *out_transaction_resume,
@@ -85,6 +80,15 @@ gboolean      ostree_repo_commit_transaction (OstreeRepo     *self,
                                               GCancellable   *cancellable,
                                               GError        **error);
 
+gboolean      ostree_repo_commit_transaction_with_stats (OstreeRepo     *self,
+                                                         guint          *out_metadata_objects_total,
+                                                         guint          *out_metadata_objects_written,
+                                                         guint          *out_content_objects_total,
+                                                         guint          *out_content_objects_written,
+                                                         guint64        *out_content_bytes_written,
+                                                         GCancellable   *cancellable,
+                                                         GError        **error);
+
 gboolean      ostree_repo_abort_transaction (OstreeRepo     *self,
                                              GCancellable   *cancellable,
                                              GError        **error);
@@ -119,8 +123,8 @@ gboolean      ostree_repo_stage_metadata_finish (OstreeRepo        *self,
 
 gboolean      ostree_repo_stage_content (OstreeRepo       *self,
                                          const char       *expected_checksum,
-                                         GInputStream     *content,
-                                         guint64           content_length,
+                                         GInputStream     *object_input,
+                                         guint64           length,
                                          guchar          **out_csum,
                                          GCancellable     *cancellable,
                                          GError          **error);
@@ -128,21 +132,21 @@ gboolean      ostree_repo_stage_content (OstreeRepo       *self,
 gboolean      ostree_repo_stage_metadata_trusted (OstreeRepo        *self,
                                                   OstreeObjectType   objtype,
                                                   const char        *checksum,
-                                                  GVariant          *object,
+                                                  GVariant          *variant,
                                                   GCancellable      *cancellable,
                                                   GError           **error);
 
 gboolean      ostree_repo_stage_content_trusted (OstreeRepo       *self,
                                                  const char       *checksum,
-                                                 GInputStream     *content,
-                                                 guint64           content_length,
+                                                 GInputStream     *object_input,
+                                                 guint64           length,
                                                  GCancellable     *cancellable,
                                                  GError          **error);
 
 void          ostree_repo_stage_content_async (OstreeRepo              *self,
                                                const char              *expected_checksum,
                                                GInputStream            *object,
-                                               guint64                  file_object_length,
+                                               guint64                  length,
                                                GCancellable            *cancellable,
                                                GAsyncReadyCallback      callback,
                                                gpointer                 user_data);
@@ -155,7 +159,7 @@ gboolean      ostree_repo_stage_content_finish (OstreeRepo        *self,
 gboolean      ostree_repo_resolve_rev (OstreeRepo  *self,
                                        const char  *refspec,
                                        gboolean     allow_noent,
-                                       char       **out_resolved,
+                                       char       **out_rev,
                                        GError     **error);
 
 gboolean      ostree_repo_write_ref (OstreeRepo  *self,
@@ -176,41 +180,81 @@ gboolean      ostree_repo_list_refs (OstreeRepo       *repo,
                                      GError          **error);
 
 gboolean      ostree_repo_load_variant_c (OstreeRepo  *self,
-                                          OstreeObjectType expected_type,
+                                          OstreeObjectType objtype,
                                           const guchar  *csum,       
                                           GVariant     **out_variant,
                                           GError       **error);
 
 gboolean      ostree_repo_load_variant (OstreeRepo  *self,
-                                        OstreeObjectType expected_type,
+                                        OstreeObjectType objtype,
                                         const char    *sha256, 
                                         GVariant     **out_variant,
                                         GError       **error);
 
 gboolean      ostree_repo_load_variant_if_exists (OstreeRepo  *self,
-                                                  OstreeObjectType expected_type,
+                                                  OstreeObjectType objtype,
                                                   const char    *sha256, 
                                                   GVariant     **out_variant,
                                                   GError       **error);
 
 gboolean ostree_repo_load_file (OstreeRepo         *self,
-                                const char         *entry_sha256,
+                                const char         *checksum,
                                 GInputStream      **out_input,
                                 GFileInfo         **out_file_info,
                                 GVariant          **out_xattrs,
                                 GCancellable       *cancellable,
                                 GError            **error);
 
+gboolean ostree_repo_load_object_stream (OstreeRepo         *self,
+                                         OstreeObjectType    objtype,
+                                         const char         *checksum,
+                                         GInputStream      **out_input,
+                                         guint64            *out_size,
+                                         GCancellable       *cancellable,
+                                         GError            **error);
+
+gboolean      ostree_repo_query_object_storage_size (OstreeRepo           *self,
+                                                     OstreeObjectType      objtype,
+                                                     const char           *sha256, 
+                                                     gsize                *out_size,
+                                                     GCancellable         *cancellable,
+                                                     GError              **error);
+
+gboolean      ostree_repo_delete_object (OstreeRepo           *self,
+                                         OstreeObjectType      objtype,
+                                         const char           *sha256, 
+                                         GCancellable         *cancellable,
+                                         GError              **error);
+
+/** 
+ * OstreeRepoCommitFilterResult:
+ * @OSTREE_REPO_COMMIT_FILTER_ALLOW: Do commit this object
+ * @OSTREE_REPO_COMMIT_FILTER_SKIP: Ignore this object
+ */
 typedef enum {
   OSTREE_REPO_COMMIT_FILTER_ALLOW,
   OSTREE_REPO_COMMIT_FILTER_SKIP
 } OstreeRepoCommitFilterResult;
 
+/**
+ * OstreeRepoCommitFilter:
+ * @repo: Repo
+ * @path: Path to file
+ * @file_info: File information
+ * @user_data: User data
+ *
+ * Returns: #OstreeRepoCommitFilterResult saying whether or not to commit this file
+ */
 typedef OstreeRepoCommitFilterResult (*OstreeRepoCommitFilter) (OstreeRepo    *repo,
                                                                 const char    *path,
                                                                 GFileInfo     *file_info,
                                                                 gpointer       user_data);
 
+/**
+ * OstreeRepoCommitModifier:
+ *
+ * A structure allowing control over commits.
+ */
 typedef struct {
   volatile gint refcount;
 
@@ -225,25 +269,28 @@ typedef struct {
 
 OstreeRepoCommitModifier *ostree_repo_commit_modifier_new (void);
 
+GType ostree_repo_commit_modifier_get_type (void);
+
+OstreeRepoCommitModifier *ostree_repo_commit_modifier_ref (OstreeRepoCommitModifier *modifier);
 void ostree_repo_commit_modifier_unref (OstreeRepoCommitModifier *modifier);
 
-gboolean      ostree_repo_stage_directory_to_mtree (OstreeRepo         *self,
-                                                    GFile              *dir,
-                                                    OstreeMutableTree  *mtree,
-                                                    OstreeRepoCommitModifier *modifier,
-                                                    GCancellable *cancellable,
-                                                    GError      **error);
+gboolean      ostree_repo_stage_directory_to_mtree (OstreeRepo                 *self,
+                                                    GFile                      *dir,
+                                                    OstreeMutableTree          *mtree,
+                                                    OstreeRepoCommitModifier   *modifier,
+                                                    GCancellable               *cancellable,
+                                                    GError                    **error);
 
-gboolean      ostree_repo_stage_archive_to_mtree (OstreeRepo         *self,
-                                                  GFile              *archive,
-                                                  OstreeMutableTree  *tree,
-                                                  OstreeRepoCommitModifier *modifier,
-                                                  gboolean            autocreate_parents,
-                                                  GCancellable *cancellable,
-                                                  GError      **error);
+gboolean      ostree_repo_stage_archive_to_mtree (OstreeRepo                   *self,
+                                                  GFile                        *archive,
+                                                  OstreeMutableTree            *tree,
+                                                  OstreeRepoCommitModifier     *modifier,
+                                                  gboolean                      autocreate_parents,
+                                                  GCancellable                 *cancellable,
+                                                  GError                      **error);
 
 gboolean      ostree_repo_stage_mtree (OstreeRepo         *self,
-                                       OstreeMutableTree  *tree,
+                                       OstreeMutableTree  *mtree,
                                        char              **out_contents_checksum,
                                        GCancellable       *cancellable,
                                        GError            **error);
@@ -253,19 +300,27 @@ gboolean      ostree_repo_stage_commit (OstreeRepo   *self,
                                         const char   *parent,
                                         const char   *subject,
                                         const char   *body,
-                                        GVariant     *metadata,
-                                        GVariant     *related_objects,
-                                        const char   *content_checksum,
-                                        const char   *metadata_checksum,
+                                        const char   *root_contents_checksum,
+                                        const char   *root_metadata_checksum,
                                         char        **out_commit,
                                         GCancellable *cancellable,
                                         GError      **error);
 
+/**
+ * OstreeRepoCheckoutMode:
+ * @OSTREE_REPO_CHECKOUT_MODE_NONE: No special options
+ * @OSTREE_REPO_CHECKOUT_MODE_USER: Ignore uid/gid of files
+ */
 typedef enum {
   OSTREE_REPO_CHECKOUT_MODE_NONE = 0,
   OSTREE_REPO_CHECKOUT_MODE_USER = 1
 } OstreeRepoCheckoutMode;
 
+/**
+ * OstreeRepoCheckoutOverwriteMode:
+ * @OSTREE_REPO_CHECKOUT_OVERWRITE_NONE: No special options
+ * @OSTREE_REPO_CHECKOUT_OVERWRITE_UNION_FILES: When layering checkouts, overwrite earlier files, but keep earlier directories
+ */
 typedef enum {
   OSTREE_REPO_CHECKOUT_OVERWRITE_NONE = 0,
   OSTREE_REPO_CHECKOUT_OVERWRITE_UNION_FILES = 1
@@ -297,6 +352,12 @@ gboolean       ostree_repo_read_commit (OstreeRepo *self,
                                         GCancellable *cancellable,
                                         GError  **error);
 
+/**
+ * OstreeRepoListObjectsFlags:
+ * @OSTREE_REPO_LIST_OBJECTS_LOOSE: List only loose (plain file) objects
+ * @OSTREE_REPO_LIST_OBJECTS_PACKED: List only packed (compacted into blobs) objects
+ * @OSTREE_REPO_LIST_OBJECTS_ALL: List all objects
+ */
 typedef enum {
   OSTREE_REPO_LIST_OBJECTS_LOOSE = (1 << 0),
   OSTREE_REPO_LIST_OBJECTS_PACKED = (1 << 1),
@@ -332,6 +393,12 @@ gboolean ostree_repo_traverse_commit (OstreeRepo         *repo,
                                       GCancellable       *cancellable,
                                       GError            **error);
 
+/**
+ * OstreeRepoPruneFlags:
+ * @OSTREE_REPO_PRUNE_FLAGS_NONE: No special options for pruning
+ * @OSTREE_REPO_PRUNE_FLAGS_NO_PRUNE: Don't actually delete objects
+ * @OSTREE_REPO_PRUNE_FLAGS_REFS_ONLY: Do not traverse individual commit objects, only follow refs
+ */
 typedef enum {
   OSTREE_REPO_PRUNE_FLAGS_NONE,
   OSTREE_REPO_PRUNE_FLAGS_NO_PRUNE,
@@ -347,12 +414,17 @@ gboolean ostree_repo_prune (OstreeRepo        *repo,
                             GCancellable      *cancellable,
                             GError           **error);
 
+/**
+ * OstreeRepoPullFlags:
+ * @OSTREE_REPO_PULL_FLAGS_NONE: No special options for pull
+ * @OSTREE_REPO_PULL_FLAGS_RELATED: Fetch objects referred to by each commit
+ */
 typedef enum {
   OSTREE_REPO_PULL_FLAGS_NONE,
   OSTREE_REPO_PULL_FLAGS_RELATED
 } OstreeRepoPullFlags;
 
-gboolean ostree_repo_pull (OstreeRepo             *repo,
+gboolean ostree_repo_pull (OstreeRepo             *self,
                            const char             *remote_name,
                            char                  **refs_to_fetch,
                            OstreeRepoPullFlags     flags,
