@@ -41,6 +41,12 @@ ot_gvariant_new_bytearray (const guchar   *data,
   return ret;
 }
 
+GVariant *
+ot_gvariant_new_ay_bytes (GBytes *bytes)
+{
+  return g_variant_new_from_bytes (G_VARIANT_TYPE ("ay"), bytes, TRUE);
+}
+
 GHashTable *
 ot_util_variant_asv_to_hash_table (GVariant *variant)
 {
@@ -96,10 +102,15 @@ ot_util_variant_take_ref (GVariant *variant)
 }
 
 /**
- * Return in @out_variant the result of memory-mapping the entire
- * contents of file @src.
+ * ot_util_variant_map:
+ * @src: a #GFile
+ * @type: Use this for variant
+ * @trusted: See documentation of g_variant_new_from_data()
+ * @out_variant: (out): Return location for new variant
+ * @error:
  *
- * Note the returned @out_variant is not floating.
+ * Memory-map @src, and store a new #GVariant referring to this memory
+ * in @out_variant.  Note the returned @out_variant is not floating.
  */
 gboolean
 ot_util_variant_map (GFile              *src,
@@ -174,12 +185,12 @@ GInputStream *
 ot_variant_read (GVariant             *variant)
 {
   GMemoryInputStream *ret = NULL;
+  gs_unref_bytes GBytes *bytes = NULL;
+
+  bytes = g_variant_get_data_as_bytes (variant);
+  ret = (GMemoryInputStream*)g_memory_input_stream_new ();
+  g_memory_input_stream_add_bytes (ret, bytes);
 
-  ret = (GMemoryInputStream*)g_memory_input_stream_new_from_data (g_variant_get_data (variant),
-                                                                  g_variant_get_size (variant),
-                                                                  NULL);
-  g_object_set_data_full ((GObject*)ret, "ot-variant-data",
-                          g_variant_ref (variant), (GDestroyNotify) g_variant_unref);
   return (GInputStream*)ret;
 }
 
