@@ -20,8 +20,7 @@
  * Author: Colin Walters <walters@verbum.org>
  */
 
-#ifndef __OT_ADMIN_FUNCTIONS__
-#define __OT_ADMIN_FUNCTIONS__
+#pragma once
 
 #include <gio/gio.h>
 #include "ostree.h"
@@ -118,12 +117,5 @@ gboolean ot_admin_get_default_ostree_dir (GFile        **out_ostree_dir,
 
 GKeyFile *ot_origin_new_from_refspec (const char *refspec);
 
-gboolean ot_admin_pull (GFile         *ostree_dir,
-                        const char    *remote,
-                        const char    *ref,
-                        GCancellable  *cancellable,
-                        GError       **error);
-
 G_END_DECLS
 
-#endif
