@@ -24,10 +24,12 @@
 
 #include "ot-builtins.h"
 #include "ostree.h"
+#include "otutil.h"
 
 #include <gio/gunixoutputstream.h>
 
 static GOptionEntry options[] = {
+  { NULL },
 };
 
 static gboolean
