@@ -24,6 +24,9 @@
 
 #include "ot-builtins.h"
 #include "ostree.h"
+#include "libgsystem.h"
+
+#include <string.h>
 
 static GOptionEntry options[] = {
   { NULL }
@@ -60,6 +63,8 @@ ostree_builtin_checksum (int argc, char **argv, GFile *repo_path_path, GCancella
   gs_unref_object GFile *f = NULL;
   AsyncChecksumData data;
 
+  memset (&data, 0, sizeof (data));
+
   context = g_option_context_new ("FILENAME - Checksum a file or directory");
   g_option_context_add_main_entries (context, options, NULL);
 
