@@ -24,39 +24,34 @@
 
 #include "ot-builtins.h"
 #include "ostree.h"
+#include "otutil.h"
 
-static char *metadata_text_path;
-static char *metadata_bin_path;
-static char *subject;
-static char *body;
-static char *parent;
-static char *branch;
-static char **metadata_strings;
-static char *statoverride_file;
-static char *opt_related_objects_file;
-static gboolean skip_if_unchanged;
-static gboolean tar_autocreate_parents;
-static gboolean no_xattrs;
-static char **trees;
-static gint owner_uid = -1;
-static gint owner_gid = -1;
+static char *opt_subject;
+static char *opt_body;
+static char *opt_branch;
+static char *opt_statoverride_file;
+static gboolean opt_link_checkout_speedup;
+static gboolean opt_skip_if_unchanged;
+static gboolean opt_tar_autocreate_parents;
+static gboolean opt_no_xattrs;
+static char **opt_trees;
+static gint opt_owner_uid = -1;
+static gint opt_owner_gid = -1;
+static gboolean opt_table_output;
 
 static GOptionEntry options[] = {
-  { "subject", 's', 0, G_OPTION_ARG_STRING, &subject, "One line subject", "subject" },
-  { "body", 'm', 0, G_OPTION_ARG_STRING, &body, "Full description", "body" },
-  { "metadata-variant-text", 0, 0, G_OPTION_ARG_FILENAME, &metadata_text_path, "File containing g_variant_print() output", "path" },
-  { "metadata-variant", 0, 0, G_OPTION_ARG_FILENAME, &metadata_bin_path, "File containing serialized variant, in host endianness", "path" },
-  { "add-metadata-string", 0, 0, G_OPTION_ARG_STRING_ARRAY, &metadata_strings, "Append given key and value (in string format) to metadata", "KEY=VALUE" },
-  { "branch", 'b', 0, G_OPTION_ARG_STRING, &branch, "Branch", "branch" },
-  { "parent", 'p', 0, G_OPTION_ARG_STRING, &parent, "Parent commit", "commit" },
-  { "tree", 0, 0, G_OPTION_ARG_STRING_ARRAY, &trees, "Overlay the given argument as a tree", "NAME" },
-  { "owner-uid", 0, 0, G_OPTION_ARG_INT, &owner_uid, "Set file ownership user id", "UID" },
-  { "owner-gid", 0, 0, G_OPTION_ARG_INT, &owner_gid, "Set file ownership group id", "GID" },
-  { "no-xattrs", 0, 0, G_OPTION_ARG_NONE, &no_xattrs, "Do not import extended attributes", NULL },
-  { "tar-autocreate-parents", 0, 0, G_OPTION_ARG_NONE, &tar_autocreate_parents, "When loading tar archives, automatically create parent directories as needed", NULL },
-  { "skip-if-unchanged", 0, 0, G_OPTION_ARG_NONE, &skip_if_unchanged, "If the contents are unchanged from previous commit, do nothing", NULL },
-  { "statoverride", 0, 0, G_OPTION_ARG_FILENAME, &statoverride_file, "File containing list of modifications to make to permissions", "path" },
-  { "related-objects-file", 0, 0, G_OPTION_ARG_FILENAME, &opt_related_objects_file, "File containing newline-separated pairs of (checksum SPACE name) of related objects", "path" },
+  { "subject", 's', 0, G_OPTION_ARG_STRING, &opt_subject, "One line subject", "subject" },
+  { "body", 'm', 0, G_OPTION_ARG_STRING, &opt_body, "Full description", "body" },
+  { "branch", 'b', 0, G_OPTION_ARG_STRING, &opt_branch, "Branch", "branch" },
+  { "tree", 0, 0, G_OPTION_ARG_STRING_ARRAY, &opt_trees, "Overlay the given argument as a tree", "NAME" },
+  { "owner-uid", 0, 0, G_OPTION_ARG_INT, &opt_owner_uid, "Set file ownership user id", "UID" },
+  { "owner-gid", 0, 0, G_OPTION_ARG_INT, &opt_owner_gid, "Set file ownership group id", "GID" },
+  { "no-xattrs", 0, 0, G_OPTION_ARG_NONE, &opt_no_xattrs, "Do not import extended attributes", NULL },
+  { "link-checkout-speedup", 0, 0, G_OPTION_ARG_NONE, &opt_link_checkout_speedup, "Optimize for commits of trees composed of hardlinks into the repository", NULL },
+  { "tar-autocreate-parents", 0, 0, G_OPTION_ARG_NONE, &opt_tar_autocreate_parents, "When loading tar archives, automatically create parent directories as needed", NULL },
+  { "skip-if-unchanged", 0, 0, G_OPTION_ARG_NONE, &opt_skip_if_unchanged, "If the contents are unchanged from previous commit, do nothing", NULL },
+  { "statoverride", 0, 0, G_OPTION_ARG_FILENAME, &opt_statoverride_file, "File containing list of modifications to make to permissions", "path" },
+  { "table-output", 0, 0, G_OPTION_ARG_NONE, &opt_table_output, "Output more information in a KEY: VALUE format", NULL },
   { NULL }
 };
 
@@ -73,7 +68,7 @@ parse_statoverride_file (GHashTable   **out_mode_add,
   gs_free char *contents = NULL;
   char **lines = NULL;
 
-  path = g_file_new_for_path (statoverride_file);
+  path = g_file_new_for_path (opt_statoverride_file);
 
   if (!g_file_load_contents (path, cancellable, &contents, &len, NULL,
                              error))
@@ -113,74 +108,6 @@ parse_statoverride_file (GHashTable   **out_mode_add,
   return ret;
 }
 
-static gboolean
-parse_related_objects_file (GVariant     **out_related_objects,
-                            GCancellable  *cancellable,
-                            GError        **error)
-{
-  gboolean ret = FALSE;
-  gsize len;
-  char **iter = NULL; /* nofree */
-  gs_unref_hashtable GHashTable *ret_hash = NULL;
-  gs_unref_variant GVariant *ret_related_objects = NULL;
-  gs_unref_object GFile *path = NULL;
-  gs_free char *contents = NULL;
-  GVariantBuilder builder;
-  gboolean builder_initialized = FALSE;
-  char **lines = NULL;
-
-  g_variant_builder_init (&builder, G_VARIANT_TYPE ("a(say)"));
-  builder_initialized = TRUE;
-
-  path = g_file_new_for_path (opt_related_objects_file);
-
-  if (!g_file_load_contents (path, cancellable, &contents, &len, NULL,
-                             error))
-    goto out;
-  
-  lines = g_strsplit (contents, "\n", -1);
-
-  for (iter = lines; iter && *iter; iter++)
-    {
-      const char *line = *iter;
-      const char *spc;
-      gs_free char *name = NULL;
-
-      if (!*line)
-        break;
-
-      spc = strchr (line, ' ');
-      if (!spc)
-        {
-          g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                       "Malformed related objects file");
-          goto out;
-        }
-
-      name = g_strndup (line, spc - line);
-
-      if (!ostree_validate_checksum_string (spc + 1, error))
-        goto out;
-
-      {
-        GVariant *csum_bytes_v = ostree_checksum_to_bytes_v (spc + 1);
-        g_variant_builder_add (&builder, "(s@ay)", name, csum_bytes_v);
-      }
-    }
-
-  ret_related_objects = g_variant_builder_end (&builder);
-  g_variant_ref_sink (ret_related_objects);
-  builder_initialized = FALSE;
-
-  ret = TRUE;
-  ot_transfer_out_value (out_related_objects, &ret_related_objects);
- out:
-  if (builder_initialized)
-    g_variant_builder_clear (&builder);
-  g_strfreev (lines);
-  return ret;
-}
-
 static OstreeRepoCommitFilterResult
 commit_filter (OstreeRepo         *self,
                const char         *path,
@@ -190,10 +117,10 @@ commit_filter (OstreeRepo         *self,
   GHashTable *mode_adds = user_data;
   gpointer value;
 
-  if (owner_uid >= 0)
-    g_file_info_set_attribute_uint32 (file_info, "unix::uid", owner_uid);
-  if (owner_gid >= 0)
-    g_file_info_set_attribute_uint32 (file_info, "unix::gid", owner_gid);
+  if (opt_owner_uid >= 0)
+    g_file_info_set_attribute_uint32 (file_info, "unix::uid", opt_owner_uid);
+  if (opt_owner_gid >= 0)
+    g_file_info_set_attribute_uint32 (file_info, "unix::gid", opt_owner_gid);
 
   if (mode_adds && g_hash_table_lookup_extended (mode_adds, path, NULL, &value))
     {
@@ -214,14 +141,16 @@ ostree_builtin_commit (int argc, char **argv, GFile *repo_path, GCancellable *ca
   gboolean ret = FALSE;
   gboolean skip_commit = FALSE;
   gboolean in_transaction = FALSE;
+  guint metadata_total = 0;
+  guint metadata_written = 0;
+  guint content_total = 0;
+  guint content_written = 0;
+  guint64 content_bytes_written = 0;
   gs_unref_object OstreeRepo *repo = NULL;
   gs_unref_object GFile *arg = NULL;
   gs_free char *parent = NULL;
   gs_free char *commit_checksum = NULL;
   gs_unref_variant GVariant *parent_commit = NULL;
-  gs_unref_variant GVariant *metadata = NULL;
-  gs_unref_variant GVariant *related_objects = NULL;
-  gs_unref_object GFile *metadata_f = NULL;
   gs_free char *contents_checksum = NULL;
   gs_unref_object OstreeMutableTree *mtree = NULL;
   gs_free char *tree_type = NULL;
@@ -231,9 +160,6 @@ ostree_builtin_commit (int argc, char **argv, GFile *repo_path, GCancellable *ca
   gs_free char *parent_content_checksum = NULL;
   gs_free char *parent_metadata_checksum = NULL;
   OstreeRepoCommitModifier *modifier = NULL;
-  GMappedFile *metadata_mappedf = NULL;
-  GVariantBuilder metadata_builder;
-  gboolean metadata_builder_initialized = FALSE;
 
   context = g_option_context_new ("[ARG] - Commit a new revision");
   g_option_context_add_main_entries (context, options, NULL);
@@ -241,119 +167,57 @@ ostree_builtin_commit (int argc, char **argv, GFile *repo_path, GCancellable *ca
   if (!g_option_context_parse (context, &argc, &argv, error))
     goto out;
 
-  if (metadata_text_path || metadata_bin_path)
-    {
-      metadata_mappedf = g_mapped_file_new (metadata_text_path ? metadata_text_path : metadata_bin_path, FALSE, error);
-      if (!metadata_mappedf)
-        goto out;
-      if (metadata_text_path)
-        {
-          metadata = g_variant_parse (G_VARIANT_TYPE ("a{sv}"),
-                                      g_mapped_file_get_contents (metadata_mappedf),
-                                      g_mapped_file_get_contents (metadata_mappedf) + g_mapped_file_get_length (metadata_mappedf),
-                                      NULL, error);
-          if (!metadata)
-            goto out;
-        }
-      else if (metadata_bin_path)
-        {
-          metadata_f = g_file_new_for_path (metadata_bin_path);
-          if (!ot_util_variant_map (metadata_f, G_VARIANT_TYPE ("a{sv}"), TRUE,
-                                    &metadata, error))
-            goto out;
-        }
-      else
-        g_assert_not_reached ();
-    }
-  else if (metadata_strings)
-    {
-      char **iter;
-
-      metadata_builder_initialized = TRUE;
-      g_variant_builder_init (&metadata_builder, G_VARIANT_TYPE ("a{sv}"));
-
-      for (iter = metadata_strings; *iter; iter++)
-        {
-          const char *s;
-          const char *eq;
-          gs_free char *key = NULL;
-
-          s = *iter;
-
-          eq = strchr (s, '=');
-          if (!eq)
-            {
-              g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                           "Missing '=' in KEY=VALUE metadata '%s'", s);
-              goto out;
-            }
-          
-          key = g_strndup (s, eq - s);
-          g_variant_builder_add (&metadata_builder, "{sv}", key,
-                                 g_variant_new_string (eq + 1));
-        }
-      metadata = g_variant_builder_end (&metadata_builder);
-      metadata_builder_initialized = FALSE;
-      g_variant_ref_sink (metadata);
-    }
-
-  if (statoverride_file)
+  if (opt_statoverride_file)
     {
       if (!parse_statoverride_file (&mode_adds, cancellable, error))
         goto out;
     }
 
-  if (opt_related_objects_file)
-    {
-      if (!parse_related_objects_file (&related_objects, cancellable, error))
-        goto out;
-    }
-
   repo = ostree_repo_new (repo_path);
   if (!ostree_repo_check (repo, error))
     goto out;
 
-  if (!branch)
+  if (!opt_branch)
     {
       g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_FAILED,
                            "A branch must be specified with --branch");
       goto out;
     }
 
-  if (!subject)
+  if (!opt_subject)
     {
       g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_FAILED,
                            "A subject must be specified with --subject");
       goto out;
     }
 
-  if (owner_uid >= 0 || owner_gid >= 0 || statoverride_file != NULL
-      || no_xattrs)
+  if (opt_owner_uid >= 0 || opt_owner_gid >= 0 || opt_statoverride_file != NULL
+      || opt_no_xattrs)
     {
       modifier = ostree_repo_commit_modifier_new ();
-      modifier->skip_xattrs = no_xattrs;
+      modifier->skip_xattrs = opt_no_xattrs;
       modifier->filter = commit_filter;
       modifier->user_data = mode_adds;
     }
 
-  if (!ostree_repo_resolve_rev (repo, branch, TRUE, &parent, error))
+  if (!ostree_repo_resolve_rev (repo, opt_branch, TRUE, &parent, error))
     goto out;
 
-  if (skip_if_unchanged && parent)
+  if (opt_skip_if_unchanged && parent)
     {
       if (!ostree_repo_load_variant (repo, OSTREE_OBJECT_TYPE_COMMIT,
                                      parent, &parent_commit, error))
         goto out;
     }
 
-  if (!ostree_repo_prepare_transaction (repo, TRUE, NULL, cancellable, error))
+  if (!ostree_repo_prepare_transaction (repo, opt_link_checkout_speedup, NULL, cancellable, error))
     goto out;
 
   in_transaction = TRUE;
 
   mtree = ostree_mutable_tree_new ();
 
-  if (argc == 1 && (trees == NULL || trees[0] == NULL))
+  if (argc == 1 && (opt_trees == NULL || opt_trees[0] == NULL))
     {
       char *current_dir = g_get_current_dir ();
       arg = g_file_new_for_path (current_dir);
@@ -369,7 +233,7 @@ ostree_builtin_commit (int argc, char **argv, GFile *repo_path, GCancellable *ca
       const char *tree;
       const char *eq;
 
-      for (tree_iter = (const char *const*)trees; *tree_iter; tree_iter++)
+      for (tree_iter = (const char *const*)opt_trees; *tree_iter; tree_iter++)
         {
           tree = *tree_iter;
 
@@ -396,7 +260,7 @@ ostree_builtin_commit (int argc, char **argv, GFile *repo_path, GCancellable *ca
             {
               arg = g_file_new_for_path (tree);
               if (!ostree_repo_stage_archive_to_mtree (repo, arg, mtree, modifier,
-                                                       tar_autocreate_parents,
+                                                       opt_tar_autocreate_parents,
                                                        cancellable, error))
                 goto out;
             }
@@ -437,7 +301,7 @@ ostree_builtin_commit (int argc, char **argv, GFile *repo_path, GCancellable *ca
   if (!ostree_repo_stage_mtree (repo, mtree, &contents_checksum, cancellable, error))
     goto out;
 
-  if (skip_if_unchanged && parent_commit)
+  if (opt_skip_if_unchanged && parent_commit)
     {
       g_variant_get_child (parent_commit, 6, "@ay", &parent_content_csum_v);
       g_variant_get_child (parent_commit, 7, "@ay", &parent_metadata_csum_v);
@@ -462,20 +326,25 @@ ostree_builtin_commit (int argc, char **argv, GFile *repo_path, GCancellable *ca
           goto out;
         }
 
-      if (!ostree_repo_stage_commit (repo, branch, parent, subject, body, metadata,
-                                     related_objects, contents_checksum, root_metadata,
+      if (!ostree_repo_stage_commit (repo, opt_branch, parent, opt_subject, opt_body,
+                                     contents_checksum, root_metadata,
                                      &commit_checksum, cancellable, error))
         goto out;
 
-      if (!ostree_repo_commit_transaction (repo, cancellable, error))
+      if (!ostree_repo_commit_transaction_with_stats (repo,
+                                                      &metadata_total,
+                                                      &metadata_written,
+                                                      &content_total,
+                                                      &content_written,
+                                                      &content_bytes_written,
+                                                      cancellable, error))
         goto out;
 
       in_transaction = FALSE;
       
-      if (!ostree_repo_write_ref (repo, NULL, branch, commit_checksum, error))
+      if (!ostree_repo_write_ref (repo, NULL, opt_branch, commit_checksum, error))
         goto out;
 
-      g_print ("%s\n", commit_checksum);
     }
   else
     {
@@ -484,7 +353,21 @@ ostree_builtin_commit (int argc, char **argv, GFile *repo_path, GCancellable *ca
 
       in_transaction = FALSE;
 
-      g_print ("%s\n", parent);
+      commit_checksum = g_strdup (parent);
+    }
+
+  if (opt_table_output)
+    {
+      g_print ("Commit: %s\n", commit_checksum);
+      g_print ("Metadata Total: %u\n", metadata_total);
+      g_print ("Metadata Written: %u\n", metadata_written);
+      g_print ("Content Total: %u\n", content_total);
+      g_print ("Content Written: %u\n", content_written);
+      g_print ("Content Bytes Written: %" G_GUINT64_FORMAT "\n", content_bytes_written);
+    }
+  else
+    {
+      g_print ("%s\n", commit_checksum);
     }
 
   ret = TRUE;
@@ -493,10 +376,6 @@ ostree_builtin_commit (int argc, char **argv, GFile *repo_path, GCancellable *ca
     {
       (void) ostree_repo_abort_transaction (repo, cancellable, NULL);
     }
-  if (metadata_builder_initialized)
-    g_variant_builder_clear (&metadata_builder);
-  if (metadata_mappedf)
-    g_mapped_file_unref (metadata_mappedf);
   if (context)
     g_option_context_free (context);
   if (modifier)
